@@ -28,14 +28,13 @@
 #include <gnome.h>
 #include <stdio.h>
 #include <sys/types.h>
-#include <getopt.h>
 #include <assert.h>
 #include <sys/errno.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <string.h>
 
-#include <error.h>
+#include <errno.h>
 
 #include "save-cwd.h" 
 #include "util.h"
@@ -65,11 +64,13 @@ static int show_listed_fs;
 /* If nonzero, use variable sized printouts instead of 512-byte blocks. */
 static int human_blocks;
 
+#ifdef NEED_UNUSED_VARS
 /* If nonzero, use 1K blocks instead of 512-byte blocks. */
 static int kilobyte_blocks;
 
 /* If nonzero, use 1M blocks instead of 512-byte blocks. */
 static int megabyte_blocks;
+#endif
 
 /* If nonzero, use the POSIX output format.  */
 static int posix_format;
@@ -125,27 +126,20 @@ static void      about_cb              (GtkWidget  *widget,
 /* GtkWidget   **t_dial; */
 
 static GnomeUIInfo file_menu[] = {
-  { GNOME_APP_UI_ITEM, N_("E_xit"), NULL, exit_menu_cb,
-    NULL, NULL, 
-    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'x', GDK_CONTROL_MASK, NULL },
-  { GNOME_APP_UI_ENDOFINFO },
+	GNOMEUIINFO_MENU_EXIT_ITEM(exit_menu_cb,NULL),
+	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo help_menu[] = {
-  GNOMEUIINFO_HELP ("help-browser"),
-  GNOMEUIINFO_SEPARATOR,
-  { GNOME_APP_UI_ITEM, N_("_About..."), NULL, about_cb, NULL, NULL,
-    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0,
-    NULL },
-  { GNOME_APP_UI_ENDOFINFO }
+	GNOMEUIINFO_HELP ("GDiskFree"),
+	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb,NULL),
+	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo app_menu[] = {
-  { GNOME_APP_UI_SUBTREE, N_("_File"), NULL, file_menu, NULL, NULL, 
-    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-  { GNOME_APP_UI_SUBTREE, N_("_Help"), NULL, help_menu, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-  { GNOME_APP_UI_ENDOFINFO },
+	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
+	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
+	GNOMEUIINFO_END
 };
 
 
@@ -200,7 +194,6 @@ build_app_window ()
   
   gnome_app_set_contents (GNOME_APP (window), w_box);
   gnome_app_create_menus (GNOME_APP (window), app_menu);
-  gtk_menu_item_right_justify (GTK_MENU_ITEM (app_menu[1].widget));
   gtk_widget_show_all (window);
 }
 
@@ -230,6 +223,7 @@ about_cb (GtkWidget *widget, gpointer data)
    For example, 8500 would be converted to 8.3M, 133456345 to 127G,
    and so on.  Numbers smaller than 1024 get the `K' suffix.  */
 
+#ifdef NEED_UNUSED_FUNCTIONS
 static char *
 human_readable_1k_blocks (int n_1k_byte_blocks, char *buf, int buf_len)
 {
@@ -271,6 +265,7 @@ human_readable_1k_blocks (int n_1k_byte_blocks, char *buf, int buf_len)
     }
   return (p);
 }
+#endif
 
 /* If FSTYPE is a type of filesystem that should be listed,
    return nonzero, else zero. */
@@ -312,8 +307,6 @@ update_dev (const char *disk, const char *mount_point, const char *fstype,
   glibtop_fsusage fsu;
   long            blocks_used;
   long            blocks_percent_used;
-  long            inodes_used;
-  long            inodes_percent_used;
   const char      *stat_file;
   gchar           *buf;
 
@@ -349,7 +342,7 @@ update_dev (const char *disk, const char *mount_point, const char *fstype,
    If FSTYPE is non-NULL, it is the type of the filesystem on DISK.
    If MOUNT_POINT is non-NULL, then DISK may be NULL -- certain systems may
    not be able to produce statistics in this case.  */
-
+#ifdef NEED_UNUSED_FUNCTIONS
 static void
 show_dev (const char *disk, const char *mount_point, const char *fstype)
 {
@@ -429,7 +422,8 @@ show_dev (const char *disk, const char *mount_point, const char *fstype)
 
   if (inode_format)
     printf (" %7ld %7ld %7ld %5ld%%",
-	    fsu.files, inodes_used, fsu.ffree, inodes_percent_used);
+	    (long)fsu.files,
+	    (long)inodes_used, (long)fsu.ffree, (long)inodes_percent_used);
   else if (human_blocks)
     {
       char buf[3][LONGEST_HUMAN_READABLE_1K_BYTE_BLOCKS + 1];
@@ -444,7 +438,7 @@ show_dev (const char *disk, const char *mount_point, const char *fstype)
     }
   else
     printf (" %7ld %7ld  %7ld  %5ld%% ",
-	    fsu.blocks, blocks_used, fsu.bavail, blocks_percent_used);
+	    (long)fsu.blocks, blocks_used, (long)fsu.bavail, blocks_percent_used);
 
   if (mount_point)
     {
@@ -461,10 +455,12 @@ show_dev (const char *disk, const char *mount_point, const char *fstype)
     }
   putchar ('\n');
 }
+#endif
 
 /* Identify the directory, if any, that device
    DISK is mounted on, and show its disk usage.  */
 
+#ifdef NEED_UNUSED_FUNCTIONS
 static void
 show_disk (const char *disk)
 {
@@ -550,10 +546,12 @@ done:
 
   return mp;
 }
+#endif
 
 /* Figure out which device file or directory POINT is mounted on
    and show its disk usage.
    STATP is the results of `stat' on POINT.  */
+#ifdef NEED_UNUSED_FUNCTION
 static void
 show_point (const char *point, const struct stat *statp)
 {
@@ -602,10 +600,12 @@ show_point (const char *point, const struct stat *statp)
       error (0, errno, "%s", point);
   }
 }
+#endif
 
 /* Determine what kind of node PATH is and show the disk usage
    for it.  STATP is the results of `stat' on PATH.  */
 
+#ifdef NEED_UNUSED_FUNCTIONS
 static void
 show_entry (const char *path, const struct stat *statp)
 {
@@ -627,6 +627,7 @@ show_all_entries (void)
     show_dev (mount_list [i].devname, mount_list [i].mountdir,
 	      mount_list [i].type);
 }
+#endif
 
 static gboolean
 update_stats (gpointer data)
@@ -645,6 +646,7 @@ update_stats (gpointer data)
 }
 /* Add FSTYPE to the list of filesystem types to display. */
 
+#ifdef NEED_UNUSED_FUNCTION
 static void
 add_fs_type (const char *fstype)
 {
@@ -655,6 +657,7 @@ add_fs_type (const char *fstype)
   fsp->fs_next = fs_select_list;
   fs_select_list = fsp;
 }
+#endif
 
 /* Add FSTYPE to the list of filesystem types to be omitted. */
 
@@ -672,8 +675,7 @@ add_excluded_fs_type (const char *fstype)
 int
 main (int argc, char **argv)
 {
-  int index, udp_timer;
-  struct stat *stats;
+  int udp_timer;
   glibtop_mountlist mountlist;
 
 
