@@ -223,67 +223,6 @@ static gint get_acoords(GtkHex *gh, gint pos, gint *x, gint *y) {
 	return TRUE;
 }
 
-/*
- * the cursor rendering stuff...
- */
-static void render_ac(GtkHex *gh) {
-	gint cx, cy;
-	
-	if(!GTK_WIDGET_REALIZED(gh->xdisp))
-		return;
-	
-	if(gh->adisp_gc == NULL) {
-		gh->adisp_gc = gdk_gc_new(gh->adisp->window);
-		gdk_gc_set_exposures(gh->adisp_gc, TRUE);
-	}
-	
-	gdk_gc_set_function(gh->adisp_gc, GDK_INVERT);
-	
-	if(get_acoords(gh, gh->cursor_pos, &cx, &cy))
-		gdk_draw_rectangle(gh->adisp->window, gh->adisp_gc, (gh->active_view == VIEW_ASCII),
-						   cx, cy, gh->char_width, gh->char_height - 1);
-
-	gdk_gc_set_function(gh->adisp_gc, GDK_COPY);
-}
-
-static void render_xc(GtkHex *gh) {
-	gint cx, cy;
-
-	if(!GTK_WIDGET_REALIZED(gh->xdisp))
-		return;
-
-	if(gh->xdisp_gc == NULL) {
-		gh->xdisp_gc = gdk_gc_new(gh->xdisp->window);
-		gdk_gc_set_exposures(gh->xdisp_gc, TRUE);
-	}
-
-	gdk_gc_set_function(gh->xdisp_gc, GDK_INVERT);
-
-	if(get_xcoords(gh, gh->cursor_pos, &cx, &cy)) {
-		if(gh->lower_nibble)
-			cx += gh->char_width;
-		gdk_draw_rectangle(gh->xdisp->window, gh->xdisp_gc, (gh->active_view == VIEW_HEX),
-						   cx, cy, gh->char_width, gh->char_height - 1);
-	}
-
-	gdk_gc_set_function(gh->xdisp_gc, GDK_COPY);
-}
-
-static void show_cursor(GtkHex *gh) {
-	if(!gh->cursor_shown) {
-		render_xc(gh);
-		render_ac(gh);
-	}
-	gh->cursor_shown = TRUE;
-}
-
-static void hide_cursor(GtkHex *gh) {
-	if(gh->cursor_shown) {
-		render_xc(gh);
-		render_ac(gh);
-	}
-	gh->cursor_shown = FALSE;
-}
 
 /*
  * renders a byte at offset pos in both displays
@@ -292,18 +231,10 @@ static void render_byte(GtkHex *gh, gint pos) {
 	gint cx, cy;
 	gchar buf[2];
 
-	if(!GTK_WIDGET_REALIZED(gh))
+	if(gh->xdisp_gc == NULL || gh->adisp_gc == NULL ||
+	   !GTK_WIDGET_MAPPED(gh->xdisp) || !GTK_WIDGET_MAPPED(gh->adisp))
 		return;
 
-	if(gh->xdisp_gc == NULL) {
-		gh->xdisp_gc = gdk_gc_new(gh->xdisp->window);
-		gdk_gc_set_exposures(gh->xdisp_gc, TRUE);
-	}
-	if(gh->adisp_gc == NULL) {
-		gh->adisp_gc = gdk_gc_new(gh->adisp->window);
-		gdk_gc_set_exposures(gh->adisp_gc, TRUE);
-	}
-
 	get_xcoords(gh, pos, &cx, &cy);
 	format_xbyte(gh, pos, buf);
 
@@ -326,6 +257,86 @@ static void render_byte(GtkHex *gh, gint pos) {
 				  cx, cy + gh->disp_font->ascent, &gh->document->buffer[pos], 1);
 }
 
+/*
+ * the cursor rendering stuff...
+ */
+static void render_ac(GtkHex *gh) {
+	gint cx, cy;
+	static guchar c[2] = "\0\0";
+	GtkStateType state;
+
+	if(!GTK_WIDGET_REALIZED(gh->adisp))
+		return;
+	
+	if(get_acoords(gh, gh->cursor_pos, &cx, &cy)) {
+		c[0] = gh->document->buffer[gh->cursor_pos];
+
+		if(gh->active_view == VIEW_ASCII)
+			state = GTK_STATE_SELECTED;
+		else
+			state = GTK_STATE_INSENSITIVE;
+
+		gtk_draw_box(GTK_WIDGET(gh)->style, gh->adisp->window,
+					 state, GTK_SHADOW_NONE,
+					 cx, cy, gh->char_width, gh->char_height - 1);
+		gtk_draw_string(GTK_WIDGET(gh)->style, gh->adisp->window,
+						state,
+						cx, cy + gh->disp_font->ascent, c);
+	}
+}
+
+static void render_xc(GtkHex *gh) {
+	gint cx, cy, i;
+	static guchar c[3];
+	GtkStateType state;
+
+	if(!GTK_WIDGET_REALIZED(gh->xdisp))
+		return;
+
+	if(get_xcoords(gh, gh->cursor_pos, &cx, &cy)) {
+		format_xbyte(gh, gh->cursor_pos, c);
+		if(gh->lower_nibble) {
+			cx += gh->char_width;
+			i = 1;
+		}
+		else {
+			c[1] = 0;
+			i = 0;
+		}
+
+		if(gh->active_view == VIEW_HEX)
+			state = GTK_STATE_SELECTED;
+		else
+			state = GTK_STATE_INSENSITIVE;
+
+		gtk_draw_box(GTK_WIDGET(gh)->style, gh->xdisp->window,
+					 state, GTK_SHADOW_NONE,
+					 cx, cy, gh->char_width, gh->char_height - 1);
+		gtk_draw_string(GTK_WIDGET(gh)->style, gh->xdisp->window,
+						state,
+						cx, cy + gh->disp_font->ascent, &c[i]);
+	}
+}
+
+static void show_cursor(GtkHex *gh) {
+	if(!gh->cursor_shown) {
+		if(gh->xdisp_gc != NULL || gh->adisp_gc != NULL ||
+		   GTK_WIDGET_MAPPED(gh->xdisp) || GTK_WIDGET_MAPPED(gh->adisp)) {
+			render_xc(gh);
+			render_ac(gh);
+		}
+		gh->cursor_shown = TRUE;
+	}
+}
+
+static void hide_cursor(GtkHex *gh) {
+	if(gh->cursor_shown) {
+		if(gh->xdisp_gc != NULL || gh->adisp_gc != NULL ||
+		   GTK_WIDGET_MAPPED(gh->xdisp) || GTK_WIDGET_MAPPED(gh->adisp))
+			render_byte(gh, gh->cursor_pos);
+		gh->cursor_shown = FALSE;
+	}
+}
 
 /*
  * when calling render_*_lines() the imin and imax arguments are the
@@ -341,11 +352,6 @@ static void render_hex_lines(GtkHex *gh, gint imin, gint imax) {
 	if( (!GTK_WIDGET_REALIZED(gh)) || (gh->cpl == 0) )
 		return;
 
-	if(gh->xdisp_gc == NULL) {
-		gh->xdisp_gc = gdk_gc_new(gh->xdisp->window);
-		gdk_gc_set_exposures(gh->xdisp_gc, TRUE);
-	}
-	
 	cursor_line = gh->cursor_pos / gh->cpl - gh->top_line;
 
 	gdk_gc_set_foreground(gh->xdisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_NORMAL]);
@@ -382,11 +388,6 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 	if( (!GTK_WIDGET_REALIZED(gh)) || (gh->cpl == 0) )
 		return;
 	
-	if(gh->adisp_gc == NULL) {
-		gh->adisp_gc = gdk_gc_new(gh->adisp->window);
-		gdk_gc_set_exposures(gh->adisp_gc, TRUE);
-	}
-	
 	cursor_line = gh->cursor_pos / gh->cpl - gh->top_line;
 	
 	gdk_gc_set_foreground(gh->adisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_NORMAL]);
@@ -415,6 +416,37 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 		render_ac(gh);
 }
 
+static void render_offsets(GtkHex *gh, gint imin, gint imax) {
+	GtkWidget *w = gh->offsets;
+	gint i, cursor_line;
+	gchar offstr[9];
+
+	if(!GTK_WIDGET_REALIZED(gh))
+		return;
+
+	if(gh->offsets_gc == NULL) {
+		gh->offsets_gc = gdk_gc_new(gh->offsets->window);
+		gdk_gc_set_exposures(gh->offsets_gc, TRUE);
+	}
+	
+	gdk_gc_set_foreground(gh->offsets_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_INSENSITIVE]);
+	gdk_draw_rectangle(w->window, gh->offsets_gc, TRUE,
+					   0, imin*gh->char_height, w->allocation.width,
+					   (imax - imin + 1)*gh->char_height);
+  
+	imax = MIN(imax, gh->vis_lines);
+	imax = MIN(imax, gh->lines - gh->top_line - 1);
+
+	gdk_gc_set_foreground(gh->offsets_gc, &GTK_WIDGET(gh)->style->text[GTK_STATE_NORMAL]);
+	
+	for(i = imin; i <= imax; i++) {
+		sprintf(offstr, "%08X", (gh->top_line + i)*gh->cpl);
+		gdk_draw_text(w->window, gh->disp_font, gh->offsets_gc,
+					  0, gh->disp_font->ascent + i*gh->char_height,
+					  offstr, 8);
+	}
+}
+
 /*
  * expose signal handlers for both displays
  */
@@ -444,19 +476,42 @@ static void ascii_expose(GtkWidget *w, GdkEventExpose *event, GtkHex *gh) {
 	render_ascii_lines(gh, imin, imax);
 }
 
+static void offsets_expose(GtkWidget *w, GdkEventExpose *event, GtkHex *gh) {
+	gint imin, imax;
+	
+	imin = (event->area.y) / gh->char_height;
+	imax = (event->area.y + event->area.height) / gh->char_height;
+	if((event->area.y + event->area.height) % gh->char_height)
+		imax++;
+
+	imax = MAX(imax, gh->vis_lines);
+	
+	render_offsets(gh, imin, imax);
+}
+
 /*
  * expose signal handler for the GtkHex itself: draws shadows around both displays
  */
 static void draw_shadow(GtkWidget *widget, GdkRectangle *area) {
 	GtkHex *gh = GTK_HEX(widget);
 	gint border = GTK_CONTAINER(widget)->border_width;
+	gint x;
 	
 	gdk_window_clear_area (widget->window, area->x, area->y,
 						   area->width, area->height);
-	
+
+	x = border;
+	if(gh->show_offsets) {
+		gtk_draw_shadow(widget->style, widget->window,
+						GTK_STATE_NORMAL, GTK_SHADOW_IN,
+						border, border, 8*gh->char_width + 2*widget_get_xt(widget),
+						widget->allocation.height - 2*border);
+		x += 8*gh->char_width + 2*widget_get_xt(widget);
+	}
+
 	gtk_draw_shadow(widget->style, widget->window,
 					GTK_STATE_NORMAL, GTK_SHADOW_IN,
-					border, border, gh->xdisp_width + 2*widget_get_xt(widget),
+					x, border, gh->xdisp_width + 2*widget_get_xt(widget),
 					widget->allocation.height - 2*border);
 	
 	gtk_draw_shadow(widget->style, widget->window,
@@ -478,11 +533,17 @@ static void recalc_displays(GtkHex *gh, guint width, guint height) {
 
 	gtk_widget_size_request(gh->scrollbar, &req);
 	
-	total_width -= 2*GTK_CONTAINER(gh)->border_width + 4*widget_get_xt(GTK_WIDGET(gh)) +
-		req.width;
-	
+	gh->xdisp_width = 1;
+	gh->adisp_width = 1;
+
+	total_width -= 2*GTK_CONTAINER(gh)->border_width +
+		4*widget_get_xt(GTK_WIDGET(gh)) + req.width;
+
+	if(gh->show_offsets)
+		total_width -= 2*widget_get_xt(GTK_WIDGET(gh)) + 8*gh->char_width;
+
 	total_cpl = total_width / gh->char_width;
-	
+
 	if((total_cpl == 0) || (total_width < 0)) {
 		gh->cpl = gh->lines = gh->vis_lines = 0;
 		return;
@@ -500,7 +561,10 @@ static void recalc_displays(GtkHex *gh, guint width, guint height) {
 		if(gh->cpl % gh->group_type == 0) /* just ended a group */
 			total_cpl--;
 	} while(total_cpl > 0);
-	
+
+	if(gh->cpl == 0)
+		return;
+
 	gh->lines = gh->document->buffer_size / gh->cpl;
 	if(gh->document->buffer_size % gh->cpl)
 		gh->lines++;
@@ -550,20 +614,12 @@ static void display_scrolled(GtkAdjustment *adj, GtkHex *gh) {
 	GdkRectangle rect;
 	GdkEvent *event;
 	
-	if((!GTK_WIDGET_REALIZED(gh)) || (!GTK_WIDGET_DRAWABLE(gh->xdisp)) ||
-	   (!GTK_WIDGET_DRAWABLE(gh->adisp))) {
+	if(gh->xdisp_gc == NULL ||
+	   gh->adisp_gc == NULL ||
+	   (!GTK_WIDGET_DRAWABLE(gh->xdisp)) ||
+	   (!GTK_WIDGET_DRAWABLE(gh->adisp)))
 		return;
-	}
-	
-	if(gh->xdisp_gc == NULL) {
-		gh->xdisp_gc = gdk_gc_new(gh->xdisp->window);
-		gdk_gc_set_exposures(gh->xdisp_gc, TRUE);
-	}
-	if(gh->adisp_gc == NULL) {
-		gh->adisp_gc = gdk_gc_new(gh->adisp->window);
-		gdk_gc_set_exposures(gh->adisp_gc, TRUE);
-	}
-	
+
 	hide_cursor(gh);
 	
 	gh->top_line = adj->value;
@@ -602,7 +658,20 @@ static void display_scrolled(GtkAdjustment *adj, GtkHex *gh) {
 						 0, dest_min,
 						 gh->adisp->allocation.width,
 						 source_max - source_min);
-		
+		if(gh->offsets) {
+			if(gh->offsets_gc == NULL) {
+				gh->offsets_gc = gdk_gc_new(gh->offsets->window);
+				gdk_gc_set_exposures(gh->offsets_gc, TRUE);
+			}
+			gdk_draw_pixmap (gh->offsets->window,
+							 gh->offsets_gc,
+							 gh->offsets->window,
+							 0, source_min,
+							 0, dest_min,
+							 gh->offsets->allocation.width,
+							 source_max - source_min);
+		}
+
 		while ((event = gdk_event_get_graphics_expose (gh->xdisp->window)) != NULL) {
 			gtk_widget_event (gh->xdisp, event);
 			if (event->expose.count == 0) {
@@ -620,11 +689,24 @@ static void display_scrolled(GtkAdjustment *adj, GtkHex *gh) {
 			}
 			gdk_event_free (event);
 		}
+
+		if(gh->offsets) {
+			while ((event = gdk_event_get_graphics_expose (gh->offsets->window)) != NULL) {
+				gtk_widget_event (gh->offsets, event);
+				if (event->expose.count == 0) {
+					gdk_event_free (event);
+					break;
+				}
+				gdk_event_free (event);
+			}
+		}
 	}
 	
 	if (rect.height != 0) {
 		gtk_widget_draw(gh->xdisp, &rect);
 		gtk_widget_draw(gh->adisp, &rect);
+		if(gh->offsets)
+			gtk_widget_draw(gh->offsets, &rect);
 	}
 	
 	show_cursor(gh);
@@ -753,7 +835,36 @@ static void ascii_motion_cb(GtkWidget *w, GdkEventMotion *event, GtkHex *gh) {
 	}
 }
 
+static void show_offsets_widget(GtkHex *gh) {
+	gh->offsets = gtk_drawing_area_new();
+	gtk_widget_set_events (gh->offsets, GDK_EXPOSURE_MASK);
+	gtk_signal_connect(GTK_OBJECT(gh->offsets), "expose_event",
+					   GTK_SIGNAL_FUNC(offsets_expose), gh);
+	gtk_fixed_put(GTK_FIXED(gh), gh->offsets, 0, 0);
+	gtk_widget_show(gh->offsets);
+}
+
+static void hide_offsets_widget(GtkHex *gh) {
+	if(gh->offsets) {
+		gtk_container_remove(GTK_CONTAINER(gh), gh->offsets);
+		gh->offsets = NULL;
+		gh->offsets_gc = NULL;
+	}
+}
+
+static gint hex_map_event(GtkWidget *widget, GdkEventAny *event, GtkHex *gh) {
+	gh->xdisp_gc = gdk_gc_new(gh->xdisp->window);
+	gdk_gc_set_exposures(gh->xdisp_gc, TRUE);
+}
+	
+static gint ascii_map_event(GtkWidget *widget, GdkEventAny *event, GtkHex *gh) {
+	gh->adisp_gc = gdk_gc_new(gh->adisp->window);
+	gdk_gc_set_exposures(gh->adisp_gc, TRUE);
+}
+
 static void gtk_hex_realize(GtkWidget *widget) {
+	GtkHex *gh = GTK_HEX(widget);
+
 	if(GTK_WIDGET_CLASS(parent_class)->realize)
 		(* GTK_WIDGET_CLASS(parent_class)->realize)(widget);  	
 
@@ -772,6 +883,11 @@ static void gtk_hex_real_data_changed(GtkHex *gh, gpointer data) {
 	
 	start_line = change_data->start/gh->cpl - gh->top_line;
 	end_line = change_data->end/gh->cpl - gh->top_line;
+
+	if(end_line < 0 ||
+	   start_line > gh->vis_lines)
+		return;
+
 	start_line = MAX(start_line, 0);
 	end_line = MIN(end_line, gh->vis_lines);
 	
@@ -907,20 +1023,25 @@ static void gtk_hex_size_allocate(GtkWidget *w, GtkAllocation *alloc) {
 	xt = widget_get_xt(w);
 	yt = widget_get_yt(w);
 
-	my_alloc.x = border_width + xt;
+   	my_alloc.x = border_width + xt;
 	my_alloc.y = border_width + yt;
-	my_alloc.height = alloc->height - 2*border_width - 2*yt;
+	my_alloc.height = MAX(alloc->height - 2*border_width - 2*yt, 1);
+	if(gh->show_offsets) {
+		my_alloc.width = 8*gh->char_width;
+		gtk_widget_size_allocate(gh->offsets, &my_alloc);
+		my_alloc.x += 2*xt + my_alloc.width;
+	}
 	my_alloc.width = gh->xdisp_width;
 	gtk_widget_size_allocate(gh->xdisp, &my_alloc);
 	my_alloc.x = alloc->width - border_width - gh->scrollbar->requisition.width;
 	my_alloc.y = border_width;
 	my_alloc.width = gh->scrollbar->requisition.width;
-	my_alloc.height = alloc->height - 2*border_width;
+	my_alloc.height = MAX(alloc->height - 2*border_width, 1);
 	gtk_widget_size_allocate(gh->scrollbar, &my_alloc);
 	my_alloc.x -= gh->adisp_width + xt;
 	my_alloc.y = border_width + yt;
 	my_alloc.width = gh->adisp_width;
-	my_alloc.height = alloc->height - 2*border_width - 2*xt;
+	my_alloc.height = MAX(alloc->height - 2*border_width - 2*yt, 1);
 	gtk_widget_size_allocate(gh->adisp, &my_alloc);
 	
 	show_cursor(gh);
@@ -1029,6 +1150,8 @@ static void gtk_hex_init(GtkHex *gh) {
 					   GTK_SIGNAL_FUNC(hex_button_cb), gh);
 	gtk_signal_connect(GTK_OBJECT(gh->xdisp), "motion_notify_event",
 					   GTK_SIGNAL_FUNC(hex_motion_cb), gh);
+	gtk_signal_connect(GTK_OBJECT(gh->xdisp), "map_event",
+					   GTK_SIGNAL_FUNC(hex_map_event), gh);
 	gtk_fixed_put(GTK_FIXED(gh), gh->xdisp, 0, 0);
 	gtk_widget_show(gh->xdisp);
 	
@@ -1043,12 +1166,14 @@ static void gtk_hex_init(GtkHex *gh) {
 					   GTK_SIGNAL_FUNC(ascii_button_cb), gh);
 	gtk_signal_connect(GTK_OBJECT(gh->adisp), "motion_notify_event",
 					   GTK_SIGNAL_FUNC(ascii_motion_cb), gh);
+	gtk_signal_connect(GTK_OBJECT(gh->adisp), "map_event",
+					   GTK_SIGNAL_FUNC(ascii_map_event), gh);
 	gtk_fixed_put(GTK_FIXED(gh), gh->adisp, 0, 0);
 	gtk_widget_show(gh->adisp);
 	
 	gtk_signal_connect(GTK_OBJECT(gh->adj), "value_changed",
 					   GTK_SIGNAL_FUNC(display_scrolled), gh);
-	
+
 	gh->scrollbar = gtk_vscrollbar_new(gh->adj);
 	gtk_fixed_put(GTK_FIXED(gh), gh->scrollbar, 0, 0);
 	gtk_widget_show(gh->scrollbar);
@@ -1170,8 +1295,10 @@ guchar gtk_hex_get_byte(GtkHex *gh, guint offset) {
  * sets data group type (see GROUP_* defines in gtkhex.h)
  */
 void gtk_hex_set_group_type(GtkHex *gh, guint gt) {
+	hide_cursor(gh);
 	gh->group_type = gt;
 	gtk_widget_queue_resize(GTK_WIDGET(gh));
+	show_cursor(gh);
 }
 
 /*
@@ -1189,3 +1316,17 @@ void gtk_hex_set_font(GtkHex *gh, GdkFont *font) {
 	
 	redraw_widget(GTK_WIDGET(gh));
 }
+
+/*
+ *  do we show the offsets of lines?
+ */
+void gtk_hex_show_offsets(GtkHex *gh, gboolean show) {
+	if(gh->show_offsets == show)
+		return;
+
+	gh->show_offsets = show;
+	if(show)
+		show_offsets_widget(gh);
+	else
+		hide_offsets_widget(gh);
+}
