@@ -52,12 +52,13 @@ typedef struct
   glibtop_swap    swap;
   glibtop_uptime  uptime;
   glibtop_loadavg loadavg;
+  glibtop_netload netload;
 }
 gtop_struct;
 #endif
 
 static char *prog_name = "gstripchart";
-static char *prog_version = "1.5";
+static char *prog_version = "1.6";
 static char *config_file = NULL;
 static float chart_interval = 5.0;
 static float chart_filter = 0.0;
@@ -65,12 +66,14 @@ static float slider_interval = 0.2;
 static float slider_filter = 0.0;
 static int include_menubar = 0;
 static int include_slider = 1;
+static int status_outline = 1;
 static int post_init = 0;
 static int minor_tick=0, major_tick=0;
 static int geometry_flags;
-static int geometry_w=160, geometry_h=100;
+static int geometry_w=200, geometry_h=50;
 static int geometry_x, geometry_y;
 static int root_width, root_height;
+static int update_counts;
 
 /*
  * streq -- case-blind string comparison returning true on equality.
@@ -289,7 +292,20 @@ Gtop_data gtop_vars[] =
   { "loadavg_1m",       'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[0]) },
   { "loadavg_5m",       'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[1]) },
   { "loadavg_15m",      'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[2]) },
+#if 0
+  /* glibtop netload stats */
+  { "net_pkts_in",      'L', &gtop_netload, GTOP_OFF(netload.packets_in)    },
+  { "net_pkts_out",     'L', &gtop_netload, GTOP_OFF(netload.packets_out)   },
+  { "net_pkts_tot",     'L', &gtop_netload, GTOP_OFF(netload.packets_total) },
 
+  { "net_bytes_in",     'L', &gtop_netload, GTOP_OFF(netload.bytes_in)      },
+  { "net_bytes_out",    'L', &gtop_netload, GTOP_OFF(netload.bytes_out)     },
+  { "net_bytes_tot",    'L', &gtop_netload, GTOP_OFF(netload.bytes_total)   },
+
+  { "net_errs_in",      'L', &gtop_netload, GTOP_OFF(netload.errors_in)     },
+  { "net_errs_out",     'L', &gtop_netload, GTOP_OFF(netload.errors_out)    },
+  { "net_errs_tot",     'L', &gtop_netload, GTOP_OFF(netload.errors_total)  },
+#endif
   /* end of array marker */
   { NULL,	         0,  NULL,          0 }
 };
@@ -355,7 +371,7 @@ num_op(Expr *e)
       if (*e->s == ')')
 	stripbl(e, 1);
       else
-	eval_error(e, "rparen expected");
+	eval_error(e, _("rparen expected"));
     }
   else if (*e->s == '$' || *e->s == '~')
     {
@@ -372,8 +388,7 @@ num_op(Expr *e)
 	{
 	  int id_num = atoi(id);
 	  if (id_num > e->vars)
-	    eval_error(e, "no such field: %d", id_num);
-	  stripbl(e, 0);
+	    eval_error(e, _("no such field: %d"), id_num);
 	  val = e->now[id_num-1];
 	  if (id_intro == '~')
 	    val -= e->last[id_num-1];
@@ -383,22 +398,27 @@ num_op(Expr *e)
 	  val = chart_interval;
 	  /* if (e->s[-1] == '~') val = 0; */
 	}
-      else if (streq(id, "t"))	/* time of day, in seconds */
+      else if (streq(id, "t"))	/* delta time, in seconds */
 	{
 	  val = e->t_diff;
 	  /* if (e->s[-1] == '~') val = 0; */
 	}
+      else if (streq(id, "u"))	/* update counts, for debugging */
+	{
+	  val = update_counts;
+	}
 #ifdef HAVE_LIBGTOP
       else if (gtop_value(id, e->gtp, &val))
 	  ; /* gtop_value handles the assignment to val */
 #endif
       else if (!*id)
-	eval_error(e, "missing variable identifer");
+	eval_error(e, _("missing variable identifer"));
       else
-	eval_error(e, "invalid variable identifer: %s", id);
+	eval_error(e, _("invalid variable identifer: %s"), id);
+      stripbl(e, 0);
     }
   else
-    eval_error(e, "number expected");
+    eval_error(e, _("number expected"));
   return val;
 }
 
@@ -475,7 +495,7 @@ static double eval(
   stripbl(&e, 0);
   e.val = add_op(&e);
   if (*e.s)
-    eval_error(&e, "extra gunk at end");
+    eval_error(&e, _("extra gunk at end: \"%s\""), e.s);
 
   return e.val;
 }
@@ -485,18 +505,22 @@ static double eval(
  */
 typedef struct
 {
-  char active;		/* set if this parameter should be displayed */
+  char active;		/* set if this parameter should be evaluated */
+  char is_led;		/* set if this is an LED display, rather than a plot */
   char id_char;		/* single-char parameter name abbreviation */
   char *ident;		/* paramater name */
-  char *color_name;	/* the name of the fg color for this parameter */
   char *filename;	/* the file to read this parameter from */
   char *pattern;	/* marks the line in the file for this paramater */
   char *eqn;		/* equation used to compute this paramater value */
   char *eqn_src;	/* where the eqn was defined, for error reporting */
   float hi, lo;		/* highest and lowest values expected */
   float top, bot;	/* highest and lowest display values */
+  char *color_name;	/* the name of the fg color for this parameter */
   GdkColor gdk_color;	/* the rgb values for the color */
-  GdkGC *gdk_gc;	/* FIX THIS:the graphics context entry for the color */
+  GdkGC *gdk_gc;	/* the graphics context entry for the color */
+  int num_leds;
+  GdkColor *led_color;	/* the rgb values for the LED colors */
+  GdkGC **led_gc;	/* the graphics context entries for the LED colors */
   int vars;		/* how many vars to read from the line in the file */
   double *last, *now;	/* the last and current vars, as enumerated above */
   float *val;		/* value history */
@@ -548,7 +572,7 @@ defns_error(char *fn, int ln, char *fmt, ...)
   va_start(args, fmt);
   fprintf(stderr, "%s: ", prog_name);
   if (fn)
-    fprintf(stderr, "%s, line %d: ", fn, ln);
+    fprintf(stderr, _("%s, line %d: "), fn, ln);
   vfprintf(stderr, fmt, args);
   fprintf(stderr, "\n");
   va_end(args);
@@ -615,7 +639,7 @@ read_param_defns(Param_glob *pgp)
 {
   int i, j, lineno = 0, params = 0;
   Param **p = NULL;
-  char fn[FILENAME_MAX], home_fn[FILENAME_MAX];
+  char fn[FILENAME_MAX], home_fn[FILENAME_MAX], etc_fn[FILENAME_MAX];
 #ifdef CONFDIR
   char conf_fn[FILENAME_MAX];
 #endif
@@ -625,7 +649,7 @@ read_param_defns(Param_glob *pgp)
     {
       strcpy(fn, config_file);
       if ((fd=fopen(fn, "r")) == NULL)
-	defns_error(NULL, 0, "can't open config file \"%s\"", config_file);
+	defns_error(NULL, 0, _("can't open config file \"%s\""), config_file);
     }
   else
     {
@@ -640,6 +664,7 @@ read_param_defns(Param_glob *pgp)
 	    {
 	      /* Try in the /etc directory. */
 	      strcpy(fn, "/etc/gstripchart.conf");
+	      strcpy(etc_fn, fn);
 	      if ((fd=fopen(fn, "r")) == NULL)
 		{
 #ifdef CONFDIR
@@ -649,6 +674,7 @@ read_param_defns(Param_glob *pgp)
 		  if ((fd=fopen(fn, "r")) == NULL)
 #endif
 		    {
+		      /* FIX THIS: i18n required. */
 		      defns_error(
 			NULL, 0,
 			"can't open config file \"%s\", \"%s\", "
@@ -656,7 +682,7 @@ read_param_defns(Param_glob *pgp)
 			"\"%s\", "
 #endif
 			"or \"%s\"", 
-			"./gstripchart.conf", home_fn, fn
+			"./gstripchart.conf", home_fn, etc_fn
 #ifdef CONFDIR
 			, conf_fn
 #endif
@@ -696,6 +722,8 @@ read_param_defns(Param_glob *pgp)
 		include_menubar = yes_no(val);
 	      else if (streq(key, "slider"))
 		include_slider = yes_no(val);
+	      else if (streq(key, "status-outline"))
+		status_outline = yes_no(val);
 	      else if (streq(key, "minor_ticks"))
 		minor_tick = atoi(val);
 	      else if (streq(key, "major_ticks"))
@@ -712,7 +740,7 @@ read_param_defns(Param_glob *pgp)
 		    display = gtk_graph;
 		  else
 		    defns_error(
-		      fn, lineno, "invalid display type: %s", val);
+		      fn, lineno, _("invalid display type: %s"), val);
 		}
 	      /* An "identifier" or "begin" keyword introduces a new
                  parameter.  We bump the params count, and allocate
@@ -725,6 +753,7 @@ read_param_defns(Param_glob *pgp)
 		  p[params-1] = malloc(sizeof(*p[params-1]));
 		  p[params-1]->ident = strdup(val);
 		  p[params-1]->active = 1;
+		  p[params-1]->is_led = 0;
 		  p[params-1]->vars = 0;
 		  p[params-1]->id_char = '*';
 		  p[params-1]->pattern = NULL;
@@ -732,6 +761,9 @@ read_param_defns(Param_glob *pgp)
 		  p[params-1]->eqn = NULL;
 		  p[params-1]->eqn_src = NULL;
 		  p[params-1]->color_name = NULL;
+		  p[params-1]->num_leds = 0;
+		  p[params-1]->led_color = NULL;
+		  p[params-1]->led_gc = NULL;
 		  p[params-1]->hi = p[params-1]->top = 1.0;
 		  p[params-1]->lo = p[params-1]->bot = 0.0;
 		}
@@ -742,14 +774,35 @@ read_param_defns(Param_glob *pgp)
 		      val, p[params-1]->ident);
 		}
 	      else if (params == 0)
-		defns_error(fn, lineno, "identifier or begin must be first");
+		defns_error(fn, lineno,_("identifier or begin must be first"));
 	      else if (streq(key, "id_char"))
 		p[params-1]->id_char = val[0];
 	      else if (streq(key, "color"))
 		{
 		  p[params-1]->color_name = strdup(val);
 		  if (!gdk_color_parse(val, &p[params-1]->gdk_color))
-		    defns_error(fn, lineno, "unrecognized color: %s", val);
+		    defns_error(fn, lineno, _("unrecognized color: %s"), val);
+		}
+	      else if (streq(key, "lights"))
+		{
+		  int n;
+		  Param *pp = p[params-1];
+		  char *t = strtok(val, ",");
+		  for (n = 0; t != NULL; n++, t = strtok(NULL, ","))
+		    {
+		      char *cname = strdup(skipbl(t));
+		      trimtb(cname);
+		      pp->led_color = realloc(
+			pp->led_color, (n+1) * sizeof(*pp->led_color));
+		      if (!gdk_color_parse(cname, &pp->led_color[n]))
+			{
+			  defns_error(
+			    fn, lineno, _("unrecognized color: %s"), cname);
+			}
+		      free(cname);
+		    }
+		  pp->num_leds = n;
+		  pp->is_led = 1;
 		}
 	      else if (streq(key, "active"))
 		p[params-1]->active = yes_no(val);
@@ -771,7 +824,7 @@ read_param_defns(Param_glob *pgp)
 	      else if (streq(key, "minimum"))
 		p[params-1]->lo = p[params-1]->bot = atof(val);
 	      else
-		defns_error(fn, lineno, "invalid option: \"%s\"", bp);
+		defns_error(fn, lineno, _("invalid option: \"%s\""), bp);
 	    }
 	}
     }
@@ -855,6 +908,7 @@ update_values(Param_glob *pgp, Param_glob *slave_pgp)
 {
   int param_num, last_val_pos, next_val_pos;
 
+  update_counts++;
   pgp->t_last = pgp->t_now;
   gettimeofday(&pgp->t_now, NULL);
   pgp->t_diff = (pgp->t_now.tv_sec - pgp->t_last.tv_sec) +
@@ -870,6 +924,10 @@ update_values(Param_glob *pgp, Param_glob *slave_pgp)
     glibtop_get_uptime(&pgp->gtop.uptime);
   if (gtop_loadavg)
     glibtop_get_loadavg(&pgp->gtop.loadavg);
+#if 0
+  if (gtop_netload)
+    glibtop_get_netload(&pgp->gtop.netload, "FIX THIS: I/F name req'd");
+#endif
 #endif
 
   last_val_pos = pgp->new_val;
@@ -908,17 +966,19 @@ update_values(Param_glob *pgp, Param_glob *slave_pgp)
 	  /* Copy now vals to last vals, update now vals, and compute
 	     a new param value based on the new now vals.  */
 	  memcpy(p->last, p->now, p->vars * sizeof(p->last[0]));
-	  split_and_extract(buf, p);
-	  val = eval(
-	    p->eqn, p->eqn_src, pgp->t_diff,
+	  if ((!p->pattern || strstr(buf, p->pattern))
+	      && p->vars <= split_and_extract(buf, p))
+	    {
+	      val = eval(
+		p->eqn, p->eqn_src, pgp->t_diff,
 #ifdef HAVE_LIBGTOP
-	    &pgp->gtop,
+		&pgp->gtop,
 #endif
-	    p->vars, p->last, p->now );
-
+		p->vars, p->last, p->now );
+	    }
 	  /* Low-pass filter the new val, and add to the val history. */
 	  prev = p->val[last_val_pos];
-	  val = p->val[next_val_pos] = prev + pgp->lpf_const * (val - prev);
+	  p->val[next_val_pos] = prev + pgp->lpf_const * (val - prev);
 	}
     }
 }
@@ -931,7 +991,7 @@ no_display(void)
 {
   while (1)
     {
-      usleep((int)(1e6 * chart_interval + 0.5));
+      usleep((int)(1e6 * chart_interval));
       update_values(&chart_glob, NULL);
     }
 }
@@ -946,7 +1006,7 @@ numeric_with_ident(void)
 
   while (1)
     {
-      usleep((int)(1e6 * chart_interval + 0.5));
+      usleep((int)(1e6 * chart_interval));
       update_values(&chart_glob, NULL);
 
       for (p=0; p<chart_glob.params; p++)
@@ -976,10 +1036,10 @@ numeric_with_graph(void)
 
   while (1)
     {
-      usleep((int)(1e6 * chart_interval + 0.5));
+      usleep((int)(1e6 * chart_interval));
       update_values(&chart_glob, NULL);
 
-      for (p=0; p<chart_glob.params; p++)
+      for (p = 0; p < chart_glob.params; p++)
 	if (chart_glob.parray[p]->active)
 	  {
 	    float v = chart_glob.parray[p]->val[chart_glob.new_val];
@@ -1008,7 +1068,7 @@ readjust_top_for_width(int width)
       /* Find the largest value that'll be visible at this width. */
       v = chart_glob.new_val;
       t = chart_glob.parray[p]->val[v];
-      for (i=1; i<n; i++)
+      for (i = 1; i < n; i++)
 	{
 	  if (--v < 0)
 	    v = chart_glob.max_val - 1;
@@ -1044,24 +1104,34 @@ static GdkColormap *colormap;
 static gint
 config_handler(GtkWidget *widget, GdkEventConfigure *e, gpointer whence)
 {
-  int p, w = widget->allocation.width, h = widget->allocation.height;
+  int p, c, w = widget->allocation.width, h = widget->allocation.height;
 
   /* On the initial configuration event, get the window colormap and
      allocate a color in it for each parameter color. */
   if (colormap == NULL)
     {
       colormap = gdk_window_get_colormap(widget->window);
-      for (p=0; p<chart_glob.params; p++)
+      for (p = 0; p < chart_glob.params; p++)
 	{
-	  gdk_color_alloc(colormap, &chart_glob.parray[p]->gdk_color);
-	  chart_glob.parray[p]->gdk_gc = gdk_gc_new(widget->window);
-	  if (include_slider)
-	    slider_glob.parray[p]->gdk_gc = chart_glob.parray[p]->gdk_gc;
-	  gdk_gc_set_foreground(
-	    chart_glob.parray[p]->gdk_gc, &chart_glob.parray[p]->gdk_color);
-	  if (include_slider)
-	    gdk_gc_set_foreground(
-	      slider_glob.parray[p]->gdk_gc, &chart_glob.parray[p]->gdk_color);
+	  Param *cgp = chart_glob.parray[p];
+	  if (cgp->is_led)
+	    {
+	      cgp->led_gc = malloc(cgp->num_leds * sizeof(*cgp->led_gc));
+	      for (c = 0; c < cgp->num_leds; c++)
+		{
+		  gdk_color_alloc(colormap, &cgp->led_color[c]);
+		  cgp->led_gc[c] = gdk_gc_new(widget->window);
+		  gdk_gc_set_foreground(cgp->led_gc[c], &cgp->led_color[c]);
+		}
+	    }
+	  else
+	    {
+	      gdk_color_alloc(colormap, &cgp->gdk_color);
+	      cgp->gdk_gc = gdk_gc_new(widget->window);
+	      gdk_gc_set_foreground(cgp->gdk_gc, &cgp->gdk_color);
+	      if (include_slider)
+		slider_glob.parray[p]->gdk_gc = cgp->gdk_gc;
+	    }
 	}
     }
 
@@ -1099,6 +1169,36 @@ overlay_tick_marks(GtkWidget *widget, int minor, int major)
       }
 }
 
+/*
+ * overlay_status_box -- draws a box in the upper-left corner of the
+ * chart window. 
+ */
+static void
+overlay_status_box(GtkWidget *widget)
+{
+  int p, x=0, y=0, s=10;
+
+  for (p = 0; p < chart_glob.params; p++)
+    {
+      Param *cgp = chart_glob.parray[p];
+      if (cgp->is_led)
+	{
+	  int c = cgp->val[chart_glob.new_val] + 0.5;
+	  if (status_outline)
+	    gdk_draw_rectangle(
+	      widget->window, widget->style->black_gc, FALSE, x,y, s,s);
+	  if (c > 0)
+	    {
+	      if (c > cgp->num_leds)
+		c = cgp->num_leds;
+	      gdk_draw_rectangle(
+		widget->window, cgp->led_gc[c-1], TRUE, x+1,y+1, s-1,s-1);
+	    }
+	  x += s;
+	}
+    }
+}
+
 /*
  * val2y -- scales a parameter value into a y coordinate value.
  */
@@ -1117,7 +1217,7 @@ chart_expose_handler(GtkWidget *widget, GdkEventExpose *event)
      window will hold.  Plot points from newest to oldest until we run
      out of data or the window is full. */
   for (p = chart_glob.params; p--; )
-    if (chart_glob.parray[p]->active)
+    if (chart_glob.parray[p]->active && !chart_glob.parray[p]->is_led)
       {
 	float top = chart_glob.parray[p]->top;
 	int n = w < chart_glob.num_val ? w : chart_glob.num_val;
@@ -1142,6 +1242,7 @@ chart_expose_handler(GtkWidget *widget, GdkEventExpose *event)
     event->area.width, event->area.height);
 
   overlay_tick_marks(widget, minor_tick, major_tick);
+  overlay_status_box(widget);
 
   return FALSE;
 }
@@ -1174,7 +1275,7 @@ chart_timer_handler(GtkWidget *widget)
 	pixmap, widget->style->bg_gc[GTK_WIDGET_STATE(widget)],
 	TRUE, w-1, 0, 1, h);
       for (p = chart_glob.params; p--; )
-	if (chart_glob.parray[p]->active)
+	if (chart_glob.parray[p]->active && !chart_glob.parray[p]->is_led)
 	  {
 	    float top = chart_glob.parray[p]->top;
 	    int i = chart_glob.new_val;
@@ -1188,6 +1289,7 @@ chart_timer_handler(GtkWidget *widget)
 	widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 
 	pixmap, 0,0, 0,0, w,h);
       overlay_tick_marks(widget, minor_tick, major_tick);
+      overlay_status_box(widget);
     }
   return TRUE;
 }
@@ -1199,7 +1301,7 @@ slider_redraw(GtkWidget *widget)
   GdkGC *bg = widget->style->bg_gc[GTK_WIDGET_STATE(widget)];
   gdk_draw_rectangle(widget->window, bg, TRUE, 0,0, w,h);
   for (p = slider_glob.params; p--; )
-    if (slider_glob.parray[p]->active)
+    if (slider_glob.parray[p]->active && !chart_glob.parray[p]->is_led)
       {
 	GdkPoint tri[3];
 	int y = val2y(
@@ -1221,11 +1323,9 @@ slider_expose_handler(GtkWidget *widget, GdkEventExpose *event)
   return FALSE;
 }
 
-static int slider_ticks;
 static gint
 slider_timer_handler(GtkWidget *widget)
 {
-  slider_ticks++;
   update_values(&slider_glob, NULL);
   slider_redraw(widget);
   return TRUE;
@@ -1240,17 +1340,6 @@ exit_callback(void)
 
 GnomeUIInfo file_menu[] =
 {
-#if 0
-  /*
-     FIX THIS: Trying to use an event handler (click_handler) as a
-     callback doesn't work.
-  */
-  {
-    GNOME_APP_UI_ITEM, N_("Params"), N_("Examine and adjust parameters"), 
-    click_handler, NULL, NULL, 
-    GNOME_APP_PIXMAP_NONE, GNOME_STOCK_MENU_EXIT, 0, 0, NULL
-  },
-#endif
   {
     GNOME_APP_UI_ITEM, N_("E_xit"), N_("Terminate the stripchart program"), 
     exit_callback, NULL, NULL, 
@@ -1270,11 +1359,11 @@ about_callback(void)
     _(prog_name), prog_version,
     _("Copyright 1998 John Kodis"),
     authors,
-    "The GNOME stripchart program plots various user-specified parameters "
-    "as a function of time.  Its main use is to chart system performance "
-    "parameters such as CPU load, CPU utilization, network traffic levels, "
-    "and the like.  Other more ingenious uses are left as an exercise for "
-    "the interested user.",
+    _("The GNOME stripchart program plots various user-specified parameters "
+      "as a function of time.  Its main use is to chart system performance "
+      "parameters such as CPU load, CPU utilization, network traffic levels, "
+      "and the like.  Other more ingenious uses are left as an exercise for "
+      "the interested user."),
     "/usr/local/share/pixmaps/gnoapp-logo.xpm");
   gtk_widget_show(about);
   return 1;
@@ -1353,43 +1442,55 @@ prefs_callback(GtkWidget *chart, gpointer unused)
   GtkWidget *dialog, *notebook, *vbox, *clist, *active, *label;
 
   notebook = gtk_notebook_new();
-/*    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP); */
-
   param_active = realloc(
     param_active, chart_glob.params * sizeof(*param_active));
   for (p = 0; p < chart_glob.params; p++)
     {
       char lo[20], hi[20], range[100];
-      char *row[2], *ttls[2] = { "Param", "Value" };
+      char *row[2], *ttls[2] = { N_("Param"), N_("Value") };
 
       clist = gtk_clist_new_with_titles(NELS(ttls), ttls);
 
-      row[0] = "Identifier"; row[1] = chart_glob.parray[p]->ident;
+      row[0] = _("Identifier"); row[1] = _(chart_glob.parray[p]->ident);
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Color"; row[1] = chart_glob.parray[p]->color_name;
+#if 0
+/* FIXME: this should be elsewhere; but I have no idea where the color
+   name come from... here are some values.
+   FIXME: and where those Busy, Load etc come from ?
+   PS: keep the #if 0, this code must never be compiled; it is just for
+   gettext to find the strings; until somebody move it to the proper
+   place.
+*/
+   { char place_holder[]={
+   N_("yellow"),N_("blue"),N_("red"),N_("green"),N_("white"),
+   N_("Busy"),N_("Load"),N_("PPP In"),N_("PPP Out"),N_("Swap")
+   };}
+
+#endif
+      row[0] = _("Color"); row[1] = _( chart_glob.parray[p]->color_name );
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Filename"; row[1] = chart_glob.parray[p]->filename;
+      row[0] = _("Filename"); row[1] = chart_glob.parray[p]->filename;
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Pattern"; row[1] = chart_glob.parray[p]->pattern;
+      row[0] = _("Pattern"); row[1] = chart_glob.parray[p]->pattern;
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Equation"; row[1] = chart_glob.parray[p]->eqn;
+      row[0] = _("Equation"); row[1] = chart_glob.parray[p]->eqn;
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Expected range"; row[1] = range;
+      row[0] = _("Expected range"); row[1] = range;
       hi_lo_fmt(chart_glob.parray[p]->lo, lo, chart_glob.parray[p]->hi, hi);
       sprintf(range, "%s ... %s", lo, hi);
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Displayed range"; row[1] = range;
+      row[0] = _("Displayed range"); row[1] = range;
       hi_lo_fmt(chart_glob.parray[p]->bot, lo, chart_glob.parray[p]->top, hi);
       sprintf(range, "%s ... %s", lo, hi);
       gtk_clist_append(GTK_CLIST(clist), row);
-      row[0] = "Current value"; row[1] = range;
+      row[0] = _("Current value"); row[1] = range;
       hi_lo_fmt(chart_glob.parray[p]->val[chart_glob.new_val], range, 0, NULL);
       gtk_clist_append(GTK_CLIST(clist), row);
 
       gtk_clist_columns_autosize(GTK_CLIST(clist));
       gtk_widget_show(clist);
 
-      param_active[p] = active = gtk_check_button_new_with_label("Active");
+      param_active[p] = active = gtk_check_button_new_with_label(_("Active"));
       gtk_toggle_button_set_active(
 	GTK_TOGGLE_BUTTON(active), chart_glob.parray[p]->active);
       gtk_widget_show(active);
@@ -1399,13 +1500,13 @@ prefs_callback(GtkWidget *chart, gpointer unused)
       gtk_box_pack_start(GTK_BOX(vbox), active, TRUE, TRUE, 0);
       gtk_widget_show(vbox);
 
-      label = gtk_label_new(chart_glob.parray[p]->ident);
+      label = gtk_label_new(_(chart_glob.parray[p]->ident));
       gtk_widget_show(label);
       gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
     }
   gtk_widget_show(notebook);
 
-  dialog = gnome_dialog_new("Gnome Stripchart Parameters",
+  dialog = gnome_dialog_new(_("Gnome Stripchart Parameters"),
     GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_APPLY,
     GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_BUTTON_HELP, NULL);
   gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(chart));
@@ -1436,7 +1537,7 @@ text_load_clist(GtkWidget *txt, GtkWidget *box)
     {
       Param *p = chart_glob.parray[n];
       char val_str[100], top_str[100];
-      char *row_strs[] = { p->ident, val_str, top_str };
+      char *row_strs[] = { _(p->ident), val_str, top_str };
       hi_lo_fmt(p->top, top_str, p->val[chart_glob.new_val], val_str);
       gtk_clist_append(GTK_CLIST(txt), row_strs);
       gtk_clist_set_foreground(GTK_CLIST(txt), n, &p->gdk_color);
@@ -1472,6 +1573,7 @@ text_popup(GtkWidget *widget, GdkEvent *event)
     }
   else
     {
+      /* FIX THIS: i18n required */
       static char *titles[] = { "Param", "Current", "Top" };
       txt = gtk_clist_new_with_titles(NELS(titles), titles);
       gtk_widget_show(txt);
@@ -1504,25 +1606,25 @@ menu_popup(GtkWidget *widget, GdkEvent *event)
     {
       menu = gtk_menu_new();
 
-      menu_item = gtk_menu_item_new_with_label("Help");
+      menu_item = gtk_menu_item_new_with_label(_("Help"));
       gtk_menu_append(GTK_MENU(menu), menu_item);
       gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
 	GTK_SIGNAL_FUNC(help_menu_action), GTK_OBJECT(widget));
       gtk_widget_show(menu_item);
 
-      menu_item = gtk_menu_item_new_with_label("About");
+      menu_item = gtk_menu_item_new_with_label(_("About"));
       gtk_menu_append(GTK_MENU(menu), menu_item);
       gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
 	GTK_SIGNAL_FUNC(about_callback), NULL);
       gtk_widget_show(menu_item);
 
-      menu_item = gtk_menu_item_new_with_label("Params");
+      menu_item = gtk_menu_item_new_with_label(_("Params"));
       gtk_menu_append(GTK_MENU(menu), menu_item);
       gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
         GTK_SIGNAL_FUNC(prefs_callback), GTK_OBJECT(widget));
       gtk_widget_show(menu_item);
 
-      menu_item = gtk_menu_item_new_with_label("Exit");
+      menu_item = gtk_menu_item_new_with_label(_("Exit"));
       gtk_menu_append(GTK_MENU(menu), menu_item);
       gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
         GTK_SIGNAL_FUNC(exit_callback), NULL);
@@ -1550,10 +1652,29 @@ click_handler(GtkWidget *widget, GdkEvent *event, gpointer unused)
     }
 }
 
-static void
-destroy_handler(GtkWidget *widget, gpointer *data)
+/*
+ * save_handler -- run in response to a "save-yourself" signal.
+ */
+static int
+save_handler(GnomeClient *client,
+  gint phase, GnomeRestartStyle restart,
+  gint shutdown, GnomeInteractStyle interact,
+  gint fast, GtkWidget *frame)
 {
-  gtk_main_quit();
+  int i = 0;
+  char *argv[20];
+
+  argv[i++] = program_invocation_name;
+  argv[i++] = "-g";
+  argv[i++] = gnome_geometry_string(frame->window);
+  if (config_file)
+    {
+      argv[i++] = "-f";
+      argv[i++] = config_file;
+    }
+  gnome_client_set_restart_command(client, i, argv);
+
+  return TRUE;
 }
 
 /*
@@ -1563,9 +1684,9 @@ destroy_handler(GtkWidget *widget, gpointer *data)
 static void
 gtk_graph(void)
 {
+  GnomeClient *client;
   GtkWidget *frame, *h_box, *drawing;
   const int slide_w=10;		/* Set the slider width. */
-  const int min_w=80, min_h=50;	/* Set the minimum width and height. */
 
   /* Create a drawing area.  Add it to the window, show it, and
      set its expose event handler. */
@@ -1577,10 +1698,10 @@ gtk_graph(void)
   h_box = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(h_box), drawing, TRUE, TRUE, 0);
 
-  gtk_signal_connect(GTK_OBJECT(drawing), "configure_event", 
-    (GtkSignalFunc)config_handler, "draw");
-  gtk_signal_connect(GTK_OBJECT(drawing), "expose_event", 
-    (GtkSignalFunc)chart_expose_handler, NULL);
+  gtk_signal_connect(GTK_OBJECT(drawing),
+    "configure_event", (GtkSignalFunc)config_handler, "draw");
+  gtk_signal_connect(GTK_OBJECT(drawing),
+    "expose_event", (GtkSignalFunc)chart_expose_handler, NULL);
   if (include_slider)
     {
       GtkWidget *sep = gtk_vseparator_new();
@@ -1594,8 +1715,8 @@ gtk_graph(void)
       gtk_widget_show(slider);
 
       gtk_widget_set_events(slider, GDK_EXPOSURE_MASK);
-      gtk_signal_connect(GTK_OBJECT(slider), "expose_event", 
-        (GtkSignalFunc)slider_expose_handler, NULL);
+      gtk_signal_connect(GTK_OBJECT(slider),
+        "expose_event", (GtkSignalFunc)slider_expose_handler, NULL);
 
       gtk_timeout_add((int)(1000 * slider_interval),
         (GtkFunction)slider_timer_handler, slider);
@@ -1605,21 +1726,18 @@ gtk_graph(void)
   /* Create a top-level window. Set the title, minimum size (_usize),
      initial size (_default_size), and establish delete and destroy
      event handlers. */
-  frame = gnome_app_new("gstripchart", "Gnome stripchart viewer");
-  gtk_widget_set_usize(frame, min_w, min_h);
+  frame = gnome_app_new(_("gstripchart"), _("Gnome stripchart viewer"));
+  gtk_widget_set_usize(frame, 1, 1); /* min_w, min_h */
   gtk_window_set_default_size(GTK_WINDOW(frame), geometry_w, geometry_h);
-  gtk_signal_connect(
-    GTK_OBJECT(frame), "destroy",
-    GTK_SIGNAL_FUNC(destroy_handler), NULL);
-  gtk_signal_connect(
-    GTK_OBJECT(frame), "delete_event",
-    GTK_SIGNAL_FUNC(destroy_handler), NULL);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
   /* Set up the pop-up menu handler.  If a mennubar was requested, set
      that up as well.  Pack the whole works into the top-level frame. */
-  gtk_signal_connect(
-    GTK_OBJECT(frame), "button_press_event", 
-    GTK_SIGNAL_FUNC(click_handler), NULL);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "button_press_event", GTK_SIGNAL_FUNC(click_handler), NULL);
   if (include_menubar)
     gnome_app_create_menus(GNOME_APP(frame), mainmenu);
   gnome_app_set_contents(GNOME_APP(frame), h_box);
@@ -1642,6 +1760,16 @@ gtk_graph(void)
   if (geometry_flags & (XValue | YValue))
     gtk_widget_set_uposition(frame, geometry_x, geometry_y);
 
+  /* Set up session management "save-yourself" signal handler. */
+  if ((client = gnome_master_client()) != NULL)
+    {
+      char cwd[PATH_MAX];
+      getcwd(cwd, sizeof(cwd));
+      gnome_client_set_current_directory(client, cwd);
+      gtk_signal_connect(GTK_OBJECT(client),
+        "save_yourself", GTK_SIGNAL_FUNC(save_handler), frame);
+    }
+
   /* Show the top-level window and enter the main event loop. */
   gtk_widget_show(frame);
   gtk_main();
@@ -1650,8 +1778,6 @@ gtk_graph(void)
 static int
 proc_arg(int opt, const char *arg)
 {
-  /*  printf("proc_arg: opt=%c (0x%02x), arg=%s\n", isgraph(c)?c:'.', opt, arg); */
-
   switch (opt)
     {
     case 'f': config_file = strdup(arg); break;
@@ -1662,8 +1788,8 @@ proc_arg(int opt, const char *arg)
     case 'M': include_menubar = 1; break;
     case 'S': include_slider = 0; break;
     case 'g':
-      geometry_flags = XParseGeometry(
-	arg, &geometry_x, &geometry_y, &geometry_w, &geometry_h);
+      geometry_flags = XParseGeometry(arg,
+	&geometry_x, &geometry_y, &geometry_w, &geometry_h);
       break;
     case 't':
       if (streq("none", arg))
@@ -1676,7 +1802,7 @@ proc_arg(int opt, const char *arg)
 	display = gtk_graph;
       else
 	{
-	  fprintf(stderr, "invalid display type: %s\n", arg);
+	  fprintf(stderr, _("invalid display type: %s\n"), arg);
 	  return -1;
 	}
     }
@@ -1723,6 +1849,7 @@ poptOption arglist[] =
   { NULL,             '\0', 0, NULL, 0 }
 };
 
+
 /*
  * main -- for the stripchart display program.
  */
@@ -1751,6 +1878,7 @@ main(int argc, char **argv)
   root_width = 1;
   if (display == gtk_graph)
     gdk_window_get_geometry(NULL, NULL, NULL, &root_width, &root_height, NULL);
+
   chart_glob.max_val = root_width;
   chart_glob.new_val = chart_glob.num_val = 0;
   chart_glob.params = read_param_defns(&chart_glob);
