@@ -47,6 +47,12 @@ struct _GnomeLessApp {
   gchar * file;     /* File being displayed, or NULL */
   GList * dialogs;  /* Any associated dialogs - need destroying when
                        the window closes. */
+  gchar *search_string;	/* the string that was last searched for */
+  gint search_position;	/* the position in the text of the last search */
+};
+
+enum {
+	TARGET_URI_LIST
 };
 
 /****************************
@@ -73,6 +79,15 @@ static void exit_cb(GtkWidget * w, gpointer data);
 static void new_app_cb(GtkWidget * w, gpointer data);
 static void fixed_cb(GtkWidget * w, gpointer data);
 
+static void drop_file(GtkWidget *widget, GdkDragContext *context,
+                      gint x, gint y, GtkSelectionData *selectionData,
+                      guint info, guint time, GnomeLessApp *app );
+static void find_dialog();
+static void find_destroy_cb(GtkWidget * w, GdkEventAny * e, gpointer data);
+static void find_cb();
+static void find_again_cb();
+static int string_search(gchar * pattern, gchar * text);
+
 /***********************************
   Globals
   ***********************************/
@@ -98,6 +113,7 @@ parse_an_arg (poptContext ctx,
               const struct poptOption *opt,
               const char *arg, void *data)
 {
+  g_print(_("Reason %d\n"), reason);
   switch (opt->val){
     
   case GEOMETRY_KEY:
@@ -276,8 +292,8 @@ static GnomeUIInfo help_menu[] = {
 
 static GnomeUIInfo file_menu[] = {
 	GNOMEUIINFO_MENU_NEW_ITEM(N_("_New Window"),
-				  N_("New text viewer window"), 
-				  new_app_cb, NULL),
+                              N_("New text viewer window"), 
+                              new_app_cb, NULL),
 	GNOMEUIINFO_MENU_OPEN_ITEM(open_cb,NULL),
 	GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as_cb,NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -286,6 +302,12 @@ static GnomeUIInfo file_menu[] = {
 	GNOMEUIINFO_END
 };
 
+static GnomeUIInfo edit_menu[] =
+{
+    GNOMEUIINFO_MENU_FIND_ITEM(find_dialog, NULL),
+    GNOMEUIINFO_MENU_FIND_AGAIN_ITEM(find_again_cb, NULL),
+    GNOMEUIINFO_END
+};
 
 static GnomeUIInfo settings_menu[] = {
 	{GNOME_APP_UI_TOGGLEITEM, N_("_Fixed Font"), 
@@ -302,17 +324,26 @@ static GnomeUIInfo settings_menu[] = {
 
 static GnomeUIInfo main_menu[] = {
 	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
+    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
 	GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
 	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
 	GNOMEUIINFO_END
 };
 
+static const GtkTargetEntry drop_types[] = {
+  { "text/uri-list", 0, TARGET_URI_LIST },
+};
+
 static void gless_new_app(const gchar * filename, const gchar * geometry,
                           gint fd)
 {
   GnomeLessApp * app;
   GtkWidget * app_box;
 
+  const gint num_drop_types = sizeof( drop_types ) / 
+    sizeof( drop_types [0] );
+
+
   gint width = 480, height = 500;
   gboolean geometry_error = FALSE;
 
@@ -343,7 +374,7 @@ static void gless_new_app(const gchar * filename, const gchar * geometry,
     }
   }
 
-  gtk_window_set_policy(GTK_WINDOW(app->app), TRUE, TRUE, TRUE);
+  gtk_window_set_policy(GTK_WINDOW(app->app), TRUE, TRUE, FALSE);
   gtk_widget_set_usize (GTK_WIDGET(app->app), width, height);
 
   gnome_app_create_menus_with_data(GNOME_APP(app->app), main_menu, app);
@@ -365,6 +396,16 @@ static void gless_new_app(const gchar * filename, const gchar * geometry,
 
   gtk_widget_show_all(app->app);
 
+  /* setup drop support */
+  gtk_drag_dest_set( app->less,
+                     GTK_DEST_DEFAULT_MOTION |
+                     GTK_DEST_DEFAULT_HIGHLIGHT |
+                     GTK_DEST_DEFAULT_DROP,
+                     drop_types, num_drop_types,
+                     GDK_ACTION_COPY );
+  gtk_signal_connect( GTK_OBJECT( app->less ), "drag_data_received",
+                      drop_file, app );
+
   app->file = NULL; 
 
   if (filename) {
@@ -385,6 +426,8 @@ static void gless_new_app(const gchar * filename, const gchar * geometry,
   }
 
   app->dialogs = NULL;
+  app->search_string = NULL;
+  app->search_position = 0;
 }
 
 static void popup_about()
@@ -403,6 +446,37 @@ static void popup_about()
   gtk_widget_show(ga);
 }
 
+static void find_dialog(GtkWidget * w, gpointer data)
+{
+    GtkWidget *find;
+    GtkWidget *entry;
+
+    const gchar *find_history = "find_history";
+
+    entry = gnome_entry_new(find_history);
+    gnome_entry_load_history(GNOME_ENTRY(entry));
+    gtk_widget_show(entry);
+
+    find = gnome_dialog_new(_("Find string"),
+			    _("Find"),
+			    GNOME_STOCK_BUTTON_CLOSE,
+			    NULL);
+
+    gtk_object_set_data(GTK_OBJECT(find), "app", data);
+
+    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(find)->vbox),
+		       entry, TRUE, TRUE, GNOME_PAD);
+    gtk_signal_connect(GTK_OBJECT(find), "delete_event",
+		       find_destroy_cb, NULL);
+    gtk_signal_connect(GTK_OBJECT(find), "clicked",
+		       find_cb, entry);
+
+    gtk_widget_show(find);
+
+    gless_add_dialog((GnomeLessApp *) data, find);
+}
+
+
 /******************************
   Misc.
   *******************************/
@@ -425,9 +499,9 @@ gless_app_show_file(GnomeLessApp * app, const gchar * filename)
 
   if ( ! g_file_exists(filename) ) {
     gchar * s;
-    s = 
-      g_strconcat(_("No such file or directory:\n"),
-                     filename, NULL);
+    s = g_strdup_printf(_("No such file or directory:\n"
+                     "%s"),
+                     filename);
     gless_show_file_error(app, s);
     g_free(s);
     return FALSE;
@@ -443,8 +517,10 @@ gless_app_show_file(GnomeLessApp * app, const gchar * filename)
       g_free(s);
       if ( ! gnome_less_show_command(GNOME_LESS(app->less), c) ) {
         gchar * err = 
-          g_strconcat(_("Failed to decompress and display the file:\n"),
-                         filename, "\n", g_unix_error_string(errno), NULL);
+          g_strdup_printf(_("Failed to decompress and display the file:\n"
+                         "%s\n"
+                         "%s"),
+                         filename, g_unix_error_string(errno));
         gless_show_file_error(app, err);
         g_free(err);
         g_free(c);
@@ -462,8 +538,11 @@ gless_app_show_file(GnomeLessApp * app, const gchar * filename)
   else {
     if ( ! gnome_less_show_file(GNOME_LESS(app->less), filename) ) {
       gchar * err = 
-        g_strconcat(_("Error loading file:\n"), filename,
-                       "\n", g_unix_error_string(errno), NULL);
+        g_strdup_printf(_("Error loading file:\n"
+                          "%s\n"
+                          "%s"),
+                          filename,
+                          g_unix_error_string(errno));
       gless_show_file_error(app, err);
       g_free(err);
       return FALSE;
@@ -476,6 +555,7 @@ gless_app_show_file(GnomeLessApp * app, const gchar * filename)
   gnome_appbar_set_default(app->appbar, g_filename_pointer(filename));
 
   app->file = g_strdup(filename);
+  app->search_position = 0;
 
   return TRUE;
 }
@@ -527,8 +607,11 @@ static gboolean gless_app_save(GnomeLessApp * app, const gchar * path)
 
   gnome_appbar_pop (app->appbar);
 
-  s = g_strconcat(_("Failed to write file:\n"), path, 
-                     "\n", g_unix_error_string(errno), NULL);
+  s = g_strdup_printf(_("Failed to write file:\n"
+                    "%s\n"
+                    "%s"),
+                    path, 
+                    g_unix_error_string(errno));
 
   gnome_app_error(GNOME_APP(app->app), s);
   
@@ -678,7 +761,7 @@ static void open_cb(GtkWidget * w, gpointer data)
 #ifdef NEED_UNUSED_FUNCTION
 static void preferences_cb(GtkWidget *w, gpointer data)
 {
-  gnome_ok_dialog("Sorry, no preferences yet.");
+  gnome_ok_dialog(_("Sorry, no preferences yet."));
 }
 #endif
 
@@ -701,5 +784,114 @@ static void fixed_cb(GtkWidget * w, gpointer data)
 }
 
 
+static void drop_file(GtkWidget *widget, GdkDragContext *context,
+                      gint x, gint y, GtkSelectionData *selectionData,
+                      guint info, guint time, GnomeLessApp *app )
+{
+	gchar *text = NULL;
+
+    switch( info ) {
+	case TARGET_URI_LIST:
+      text = selectionData->data + strlen( "file:" );
+      /* get rid of the \r\n from the end */
+      *(text + strlen( text ) - 2) = 0;
+      gless_app_show_file(app, text);
+      break;
+    }
+    
+}
 
 
+static void find_destroy_cb(GtkWidget * w, GdkEventAny * e, gpointer data)
+{
+    GnomeLessApp *app;
+
+    app = (GnomeLessApp *) gtk_object_get_data(GTK_OBJECT(w), "app");
+
+    gless_remove_dialog(app, w);
+}
+
+static void find_cb(GtkWidget * w, gint button, gpointer data)
+{
+    GnomeLessApp *app;
+    GtkWidget *entry;
+
+    gchar *string;
+
+    app = (GnomeLessApp *) gtk_object_get_data(GTK_OBJECT(w), "app");
+    entry = gnome_entry_gtk_entry(GNOME_ENTRY(data));
+
+    string = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+
+    find_destroy_cb(w, NULL, NULL);
+
+    if (button == 0) {
+	g_free(app->search_string);
+	app->search_string = string;
+	find_again_cb(NULL, app);
+    } else {
+	g_free(string);
+    }
+}
+
+static void find_again_cb(GtkWidget * w, gpointer data)
+{
+    GnomeLessApp *app;
+    GtkText *t;
+
+    gchar *pattern;
+    gchar *text;
+
+    gint len;
+
+    gint result;
+
+    app = (GnomeLessApp *) data;
+
+    gnome_appbar_push(app->appbar, _("Searching..."));
+
+    t = ((GnomeLess *) app->less)->text;
+
+    pattern = app->search_string;
+
+    len = gtk_text_get_length(t);
+    text = gtk_editable_get_chars(GTK_EDITABLE(t), 0, len);
+
+    if(!pattern) {
+	gnome_appbar_pop(app->appbar);
+	gnome_app_flash(app->app, _("No search string was specified!"));
+        return;
+    }
+
+    result = string_search(pattern, text + app->search_position);
+
+    if (result != -1) {
+	result += app->search_position;
+	app->search_position = result + strlen(pattern);
+	gtk_editable_set_position(GTK_EDITABLE(t), result);
+	gtk_editable_select_region(GTK_EDITABLE(t), result, app->search_position);
+    } else {
+	app->search_position = 0;
+    }
+
+    gnome_appbar_pop(app->appbar);
+}
+
+static int string_search(gchar * pattern, gchar * text)
+{
+    gint i, j, k;
+    gint M = strlen(pattern);
+    gint N = strlen(text);
+
+    for (i = 0, j = M - 1; j >= 0; j--) {
+	while (pattern[j] != text[i + j]) {
+	    for (k = 0; j - k >= 0 && pattern[j - k] != text[i + j]; k++) {
+	    }
+	    i += k;
+	    j = M - 1;
+	    if ((i + j) > N)
+		return -1;
+	}
+    }
+    return i;
+}
