@@ -28,6 +28,7 @@
 
 #define DEBIAN_STRING "Debian GNU/Linux"
 #define REDHAT_STRING "Red Hat Linux"
+#define SUSE_STRING "SuSE Linux"
 
 /************************************
   Globals
@@ -121,9 +122,9 @@ static void get_portable_info()
   info[si_host] = g_strdup(buf.nodename);
   /*  info[si_domain] = g_strdup(buf.domainname); */ /* man page wrong? */
 
-  info[si_user] = g_strdup(getlogin());
+  info[si_user] = g_strdup(g_get_user_name());
 
-  info[si_display] = g_strdup(getenv("DISPLAY")); /* hmm */
+  info[si_display] = g_strdup(gdk_get_display());
 
   get_uptime();
   
@@ -153,13 +154,26 @@ static void get_linux_info()
     f = fopen("/etc/redhat-release", "r");
     if (f) {
       fgets(buf, 79, f);
+/* drmike - this seems unecessary in RH 5.2 and later
       if (strchr(buf, ' '))
         info[si_distribution_version] = g_strdup(strchr(buf, ' ')+1);
       else
+*/
         info[si_distribution_version] = g_strdup(buf);
       fclose(f);
     }
-  }
+  } else if (g_file_exists("/etc/SuSE-release")) {
+    FILE *f;
+	gchar buf[80];
+
+	info[si_distribution] = g_strdup(SUSE_STRING);
+	f = fopen("/etc/SuSE-release", "r");
+	if (f) { 
+	  fgets(buf, 79, f);
+	  info[si_distribution_version] = g_strdup(buf);
+	  fclose(f);
+	}
+  }		
   /* OK, people using other dists will need to add theirs, I have
      no idea how to identify a Red Hat system. */
   else {
